@@ -11,13 +11,14 @@ from memfault_cli.context import MemfaultCliClickContext
 from memfault_cli.deploy import Deployer
 from memfault_cli.upload import (
     AndroidAppSymbolsUploader,
+    AospSymbolDirectoryUploader,
+    AospSymbolUploader,
     BugreportUploader,
     CoredumpUploader,
+    McuSdkElfSymbolUploader,
     ReleaseArtifactUploader,
-    SoftwareArtifactUploader,
-    SymbolDirectoryUploader,
-    SymbolUploader,
     Uploader,
+    XedUploader,
 )
 
 from . import __version__
@@ -84,14 +85,34 @@ def _do_upload_or_raise(ctx: MemfaultCliClickContext, path: str, uploader_cls: T
 @click_argument_path
 @pass_memfault_cli_context
 def upload_coredump(ctx: MemfaultCliClickContext, path: str):
-    """Upload a firmware coredump for analysis.
+    """Upload an MCU coredump for analysis.
 
-    Coredumps can be added to a firmware platform by integrating the Memfault C SDK:
+    Coredumps can be added to an MCU platform by integrating the Memfault C SDK:
     https://github.com/memfault/memfault-firmware-sdk
     """
     _do_upload_or_raise(ctx, path, CoredumpUploader)
 
 
+@cli.command(name="upload-xed")
+@click.option(
+    "--hardware-version",
+    required=True,
+    help="Required for MCU builds, see https://mflt.io/34PyNGQ",
+)
+@click.option(
+    "--device-serial",
+    required=True,
+    help="The unique identifier of a device",
+)
+@click_argument_path
+@pass_memfault_cli_context
+def upload_xed(ctx: MemfaultCliClickContext, path: str, **kwargs):
+    """Upload an .xed file for analysis."""
+    ctx.obj.update(**kwargs)
+
+    _do_upload_or_raise(ctx, path, XedUploader)
+
+
 @cli.command(name="upload-bugreport")
 @click_argument_path
 @pass_memfault_cli_context
@@ -105,52 +126,111 @@ def upload_bugreport(ctx: MemfaultCliClickContext, path: str):
 @click.option(
     "--software-type",
     required=False,
-    help="Required for firmware builds, see https://mflt.io/34PyNGQ",
+    help="Required for MCU builds, see https://mflt.io/34PyNGQ",
 )
 @click.option(
     "--software-version",
     required=False,
-    help="Required for firmware builds, see https://mflt.io/34PyNGQ",
+    help="Required for MCU builds, see https://mflt.io/34PyNGQ",
 )
 @click_option_revision
 @click_option_concurrency
 @click_argument_path
 @pass_memfault_cli_context
 def upload_symbols(ctx: MemfaultCliClickContext, path: str, **kwargs):
-    """Upload symbols for a Firmware or Android build.
+    """[DEPRECATED] Upload symbols for an MCU or Android build.
+
+    Please use upload-aosp-symbols or upload-mcu-symbols instead!
+    """
+    ctx.obj.update(**kwargs)
+
+    if os.path.isdir(path):
+        uploader = AospSymbolDirectoryUploader
+    else:
+        if ctx.software_info is None:
+            uploader = AospSymbolUploader
+        else:
+            uploader = McuSdkElfSymbolUploader
+
+    _do_upload_or_raise(ctx, path, uploader)
+
+
+@cli.command(name="upload-mcu-symbols")
+@click.option(
+    "--software-type",
+    required=False,
+    help="Required for MCU symbols without Build Id, see https://mflt.io/symbol-file-build-ids",
+)
+@click.option(
+    "--software-version",
+    required=False,
+    help="Required for MCU symbols without Build Id, see https://mflt.io/symbol-file-build-ids",
+)
+@click_option_revision
+@click_argument_path
+@pass_memfault_cli_context
+def upload_mcu_symbols(ctx: MemfaultCliClickContext, path: str, **kwargs):
+    """Upload symbols for an MCU build.
+
+    Memfault will use the Build Id in the symbol file to identify it and match
+    it with data sent by that build. See https://mflt.io/symbol-file-build-ids
+    for more information on enabling Build Id in your project.
+
+    In case a Build Id cannot be found in the symbol file, a `--software-type`
+    and `--software-version` must be passed. This will then be used as
+    identifying information instead of a Build Id.
+
+    Even if a Build Id is available, it is possible to pass a `--software-type`
+    and `--software-version`. This will link the symbol file with the specified
+    Software Version, which can be useful to easily retrieve the symbol file by
+    Software Version through the Memfault UI.
 
     \b
-    Example Firmware Symbol Upload:
+    Example MCU Symbols Upload:
     \b
-        $ memfault --email bot@example.com --password user-api-key \\
+        $ memfault --org-token $ORG_TOKEN \\
                    --org acme-inc --project smart-sink \\
-            upload-symbols \\
+            upload-mcu-symbols \\
+            build/symbols.elf
+
+    \b
+    Example MCU Symbols Upload, associating it with a Software Version:
+    \b
+        $ memfault --org-token $ORG_TOKEN \\
+                   --org acme-inc --project smart-sink \\
+            upload-mcu-symbols \\
             --software-type stm32-fw \\
             --software-version 1.0.0-alpha \\
             --revision 89335ffade90ff7697e2ce5238bd4c68978b6d6e \\
             build/symbols.elf
+    """
+    ctx.obj.update(**kwargs)
+
+    _do_upload_or_raise(ctx, path, McuSdkElfSymbolUploader)
+
+
+@cli.command(name="upload-aosp-symbols")
+@click_option_concurrency
+@click_argument_path
+@pass_memfault_cli_context
+def upload_aosp_symbols(ctx: MemfaultCliClickContext, path: str, **kwargs):
+    """Upload symbols for an Android OS/AOSP build.
 
     \b
-    Example Android Symbol Upload:
+    Example AOSP Symbol Upload:
     \b
-        $ memfault --email bot@example.com --password user-api-key \\
+        $ memfault --org-token $ORG_TOKEN \\
                    --org acme-inc --project smart-sink
-            upload-symbols \\
+            upload-aosp-symbols \\
             out/target/product/generic/symbols
 
         Reference: https://mflt.io/android-os-symbol-files
     """
     ctx.obj.update(**kwargs)
 
-    if os.path.isdir(path):
-        uploader = SymbolDirectoryUploader
-    else:
-        if ctx.software_info is None:
-            uploader = SymbolUploader
-        else:
-            uploader = SoftwareArtifactUploader
-
-    _do_upload_or_raise(ctx, path, uploader)
+    _do_upload_or_raise(
+        ctx, path, AospSymbolDirectoryUploader if os.path.isdir(path) else AospSymbolUploader
+    )
 
 
 @cli.command(name="upload-android-app-symbols")
@@ -215,7 +295,7 @@ def upload_android_app_symbols(ctx: MemfaultCliClickContext, path: str, **kwargs
 @click_argument_path
 @pass_memfault_cli_context
 def upload_ota_payload(ctx: MemfaultCliClickContext, path: str, **kwargs):
-    """Upload a binary to be used for a firmware update.
+    """Upload a binary to be used for an OTA update.
 
     See https://mflt.io/34PyNGQ for details about 'hardware-version', 'software-type' and
     'software-version' nomenclature.
@@ -224,9 +304,9 @@ def upload_ota_payload(ctx: MemfaultCliClickContext, path: str, **kwargs):
     which can be used for an Over The Air (OTA) update.
 
     \b
-    Example Firmware OTA Upload:
+    Example OTA Upload:
     \b
-        $ memfault --email bot@example.com --password user-api-key \\
+        $ memfault --org-token $ORG_TOKEN \\
                    --org acme-inc --project smart-sink \\
             upload-ota-payload \\
             --hardware-version mp \\
@@ -258,9 +338,9 @@ def deploy_release(
     """Publish a Release to a Cohort.
 
     \b
-    Example Firmware OTA Upload:
+    Example Release Deployment:
     \b
-        $ memfault --email bot@example.com --password user-api-key \\
+        $ memfault --org-token $ORG_TOKEN \\
                    --org acme-inc --project smart-sink \\
                    deploy-release \\
                    --release-version 1.0.0-alpha \\
