@@ -20,6 +20,12 @@ class SoftwareInfo:  # noqa: B903 - no data classes in Python 3.6
         self.revision = revision
 
 
+class XedInfo:  # noqa: B903 - no data classes in Python 3.6
+    def __init__(self, device_serial: str, hardware_version: str):
+        self.device_serial = device_serial
+        self.hardware_version = hardware_version
+
+
 class MemfaultCliClickContext(object):
     """
     A context passed around between the memfault cli sub-commands.
@@ -31,7 +37,7 @@ class MemfaultCliClickContext(object):
     and errors can be raised in a uniform way
     """
 
-    def __init__(self):
+    def __init__(self) -> None:
         self.obj = {}
 
     def _find_obj_or_raise(self, name):
@@ -84,6 +90,12 @@ class MemfaultCliClickContext(object):
         sw_type = self.obj.get("software_type")
         sw_ver = self.obj.get("software_version")
         revision = self.obj.get("revision")
+
+        if revision and (sw_ver is None or sw_type is None):
+            raise click.exceptions.UsageError(
+                '"--software-version" and "--software-type" must be specified when using --revision'
+            )
+
         if sw_type is None and sw_ver is None:
             return None
 
@@ -94,6 +106,15 @@ class MemfaultCliClickContext(object):
 
         return SoftwareInfo(software_type=sw_type, software_version=sw_ver, revision=revision)
 
+    @property
+    def xed_info(self) -> XedInfo:
+        device_serial = self._find_obj_or_raise("device_serial")
+        hardware_version = self._find_obj_or_raise("hardware_version")
+        return XedInfo(
+            device_serial=device_serial,
+            hardware_version=hardware_version,
+        )
+
     @property
     def file_url(self) -> str:
         FILES_BASE_URL = "https://files.memfault.com"
