@@ -12,6 +12,7 @@ from zipfile import is_zipfile
 import click
 import requests
 from elftools.elf.elffile import ELFFile
+from mflt_build_id import BuildIdInspectorAndPatcher
 from requests import Response, Session
 from requests.adapters import HTTPAdapter
 from tqdm import tqdm
@@ -79,11 +80,6 @@ class Uploader:
         session.mount("http://", adapter)
         return session
 
-    @staticmethod
-    @abstractmethod
-    def project_key_auth() -> bool:
-        pass
-
     @property
     def base_url(self):
         return self.ctx.file_url
@@ -124,11 +120,6 @@ class Uploader:
         """The UI URL at which results from the operation can be seen"""
         return None
 
-    @abstractmethod
-    def requests_auth_params(self) -> dict:
-        """A dict of `requests` parameters that should be used for authentication."""
-        pass
-
     def _is_already_uploaded(self) -> bool:
         return False
 
@@ -174,18 +165,6 @@ class Uploader:
 
         check_response(response)
 
-    def _is_elf(self) -> bool:
-        with open(self.file_path, "rb") as file:
-            if file.read(4) != b"\x7FELF":
-                return False
-
-        return True
-
-    def _get_gnu_build_id_and_has_debug_info(self) -> Tuple[Optional[str], bool]:
-        with open(self.file_path, "rb") as f:
-            elf = ELFFile(f)
-            return get_gnu_build_id(elf), elf.has_dwarf_info()
-
     def upload(self, *, progressbar=True) -> bool:
         if self._is_already_uploaded():
             LOG.info(f"{self.file_path}: skipping, already uploaded.")
@@ -239,9 +218,39 @@ class CoredumpUploader(Uploader):
             return f"{self._projects_base_url()}/coredumps"
 
 
-class SymbolUploader(Uploader):
+class XedUploader(Uploader):
+    def can_upload_file(self) -> bool:
+        return True
+
+    def entity_url(self) -> str:
+        if self.authenticator.project_key_auth():
+            return f"{self._api_base_url()}/upload/qc/xed"
+        else:
+            return f"{self._projects_base_url()}/qc/xed"
+
+    def post_token_extra_request_data(self) -> Optional[Dict]:
+        info = self.ctx.xed_info
+        return {
+            "device_serial": info.device_serial,
+            "hardware_version": info.hardware_version,
+        }
+
+
+class AospSymbolUploader(Uploader):
     gnu_build_id: str = ""
 
+    def _is_elf(self) -> bool:
+        with open(self.file_path, "rb") as file:
+            if file.read(4) != b"\x7FELF":
+                return False
+
+        return True
+
+    def _get_gnu_build_id_and_has_debug_info(self) -> Tuple[Optional[str], bool]:
+        with open(self.file_path, "rb") as f:
+            elf = ELFFile(f)
+            return get_gnu_build_id(elf), elf.has_dwarf_info()
+
     def can_upload_file(self) -> Optional[Any]:
         if not self._is_elf():
             LOG.info(f"{self.file_path}: Not an ELF file")
@@ -276,24 +285,29 @@ class SymbolUploader(Uploader):
         return f"{self._projects_base_url()}/symbols"
 
 
-class SoftwareArtifactUploader(Uploader):
-    def can_upload_file(self) -> Optional[Any]:
-        if not self._is_elf():
-            LOG.info(f"{self.file_path}: Not an ELF file")
-            return False
+class McuSdkElfSymbolUploader(Uploader):
+    def can_upload_file(self) -> bool:
+        with open(self.file_path, "rb") as file:
+            if file.read(4) != b"\x7FELF":
+                LOG.info(f"{self.file_path}: Not an ELF file")
+                return False
+            file.seek(0)
 
-        self.gnu_build_id, has_debug_info = self._get_gnu_build_id_and_has_debug_info()
-        if not self.gnu_build_id:
-            # We can handle a symbol artifact without a Build ID but might as well
-            # recommend it!
-            LOG.info(
-                f"{self.file_path}: Consider adding a GNU Build ID to uniquely identify the build"
-            )
+            inspector = BuildIdInspectorAndPatcher(file)
+            if not inspector.elf.has_dwarf_info():
+                LOG.info(f"{self.file_path}: looks like an ELF but it has no .debug_info")
+                return False
 
-        if not has_debug_info:
-            LOG.info(f"{self.file_path}: looks like an ELF but it has no .debug_info")
-            return False
-        LOG.info(f"{self.file_path}: ELF file with .debug_info")
+            mcu_build_id_type, mcu_build_id, mcu_build_id_short_len = inspector.get_build_info()
+
+            if not self.ctx.software_info and mcu_build_id is None:
+                LOG.error(
+                    f"{self.file_path}: Build Id missing. Specify --software-version and --software-type options "
+                    f"or add a Build Id (see https://mflt.io/symbol-file-build-ids)"
+                )
+                return False
+
+        LOG.info(f"{self.file_path}: ELF file with .debug_info. Build Id: {mcu_build_id}")
         return True
 
     def _is_already_uploaded(self) -> bool:
@@ -302,7 +316,10 @@ class SoftwareArtifactUploader(Uploader):
     def entity_url(self) -> str:
         return f"{self._projects_base_url()}/symbols"
 
-    def ui_url(self) -> str:
+    def ui_url(self) -> Optional[str]:
+        if not self.ctx.software_info:
+            return None
+
         software_type = url_quote(self.ctx.software_info.software_type)
         software_version = url_quote(self.ctx.software_info.software_version)
         return (
@@ -310,6 +327,9 @@ class SoftwareArtifactUploader(Uploader):
         )
 
     def post_token_extra_request_data(self) -> Optional[Dict]:
+        if not self.ctx.software_info:
+            return None
+
         info = self.ctx.software_info
         return {
             "software_version": {
@@ -319,10 +339,6 @@ class SoftwareArtifactUploader(Uploader):
             },
         }
 
-    @staticmethod
-    def project_key_auth() -> bool:
-        return False
-
 
 class ReleaseArtifactUploader(Uploader):
     def can_upload_file(self) -> Optional[Any]:
@@ -396,9 +412,12 @@ class DirectoryUploader(Uploader):
 
         return did_upload
 
+    def entity_url(self) -> str:
+        raise NotImplementedError
 
-class SymbolDirectoryUploader(DirectoryUploader):
-    uploader_cls = SymbolUploader
+
+class AospSymbolDirectoryUploader(DirectoryUploader):
+    uploader_cls = AospSymbolUploader
 
 
 class ProguardMappingUploader(Uploader):
@@ -498,7 +517,7 @@ class AndroidAppSymbolsUploader(Uploader):
         )
         if not os.path.isdir(cmake_path_out):
             return False
-        sym_dir_uploader = SymbolDirectoryUploader(
+        sym_dir_uploader = AospSymbolDirectoryUploader(
             ctx=self.ctx, file_path=cmake_path_out, authenticator=self.authenticator
         )
         return sym_dir_uploader.upload()
